using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;

using Telerik.WebControls;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Helps find controls on a ribbon based page returning strongly typed objects.
	/// </summary>
	public class RibbonControlFinder : ControlFinder
	{

		#region Fields

		private Control _owner = null;
		private ApplicationBar _applicationBar = null;

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the RibbonControlFinder class.
		/// </summary>
		/// <param name="owner">
		/// The control that the finder is to be associated with.
		/// </param>
		public RibbonControlFinder(Control owner)
		{
			if (owner == null)
			{
				throw new ArgumentNullException("owner");
			}

			_owner = owner;
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets the control that the finder is associated with.
		/// </summary>
		public Control Owner
		{
			get
			{
				return _owner;
			}
		}

		#endregion Properties

		#region Methods

		/// <summary>
		/// Gets the application bar.
		/// </summary>
		/// <returns>
		/// The application bar if available; otherwise null.
		/// </returns>
		public ApplicationBar FindApplicationBar()
		{
			FindApplicationBar(this.Owner);

			return _applicationBar;
		}

		/// <summary>
		/// Gets the application menu.
		/// </summary>
		/// <returns>
		/// The application menu if available; otherwise null.
		/// </returns>
		public RadMenu FindApplicationMenu()
		{
			RadMenu applicationMenu = null;
			ApplicationBar applicationBar = FindApplicationBar();

			if ((applicationBar != null) && (!string.IsNullOrEmpty(applicationBar.ApplicationMenuID)))
			{
				applicationMenu =
					FindUniqueControl(this.Owner, applicationBar.ApplicationMenuID) as RadMenu;
			}

			return applicationMenu;
		}

		/// <summary>
		/// Gets the main tab strip as specified by the Application Bar.
		/// </summary>
		/// <returns>
		/// The tab strip if available; otherwise null.
		/// </returns>
		public RadTabStrip FindRibbonTabStrip()
		{
			RadTabStrip ribbonTabStrip = null;
			ApplicationBar applicationBar = FindApplicationBar();

			if ((applicationBar != null) && (!string.IsNullOrEmpty(applicationBar.RibbonTabStripID)))
			{
				ribbonTabStrip =
					FindUniqueControl(this.Owner, applicationBar.RibbonTabStripID) as RadTabStrip;
			}

			return ribbonTabStrip;
		}

		/// <summary>
		/// Gets a quick access button from the application bar with the specified ID.
		/// </summary>
		/// <param name="id">
		/// The ID of the button to get.
		/// </param>
		/// <returns>
		/// The button if available; otherwise null.
		/// </returns>
		public QuickAccessRibbonButton FindQuickAccessRibbonButton(string id)
		{
			QuickAccessRibbonButton button = null;
			ApplicationBar applicationBar = FindApplicationBar();

			if (applicationBar != null)
			{
				button = FindUniqueControl(applicationBar, id) as QuickAccessRibbonButton;
			}

			return button;
		}

		/// <summary>
		/// Gets a group with the specified ID.
		/// </summary>
		/// <param name="id">
		/// The ID of the group to get.
		/// </param>
		/// <returns>
		/// The group if available; otherwise null.
		/// </returns>
		public RibbonGroup FindRibbonGroup(string id)
		{
			RibbonGroup group = FindUniqueControl(this.Owner, id) as RibbonGroup;

			return group;
		}

		/// <summary>
		/// Gets a ribbon button with the specified ID.
		/// </summary>
		/// <param name="id">
		/// The ID of the button to get.
		/// </param>
		/// <returns>
		/// The button if available; otherwise null.
		/// </returns>
		public RibbonButton FindRibbonButton(string id)
		{
			RibbonButton button = FindUniqueControl(this.Owner, id) as RibbonButton;

			return button;
		}

		/// <summary>
		/// Gets a large ribbon button with the specified ID.
		/// </summary>
		/// <param name="id">
		/// The ID of the button to get.
		/// </param>
		/// <returns>
		/// The button if available; otherwise null.
		/// </returns>
		public LargeRibbonButton FindLargeRibbonButton(string id)
		{
			LargeRibbonButton button = FindRibbonButton(id) as LargeRibbonButton;

			return button;
		}

		/// <summary>
		/// Gets a small ribbon button with the specified ID.
		/// </summary>
		/// <param name="id">
		/// The ID of the button to get.
		/// </param>
		/// <returns>
		/// The button if available; otherwise null.
		/// </returns>
		public SmallRibbonButton FindSmallRibbonButton(string id)
		{
			SmallRibbonButton button = FindRibbonButton(id) as SmallRibbonButton;

			return button;
		}

		#endregion Methods

		#region Implementation Methods

		/// <summary>
		/// Finds the application bar and sets the class level variable for future reference.
		/// </summary>
		/// <param name="owner">
		/// The owning control to search through.
		/// </param>
		public void FindApplicationBar(Control owner)
		{
			if (_applicationBar == null)
			{
				foreach (Control control in owner.Controls)
				{
					if (control is ApplicationBar)
					{
						_applicationBar = (ApplicationBar) control;
					}
					else
					{
						if (control.HasControls())
						{
							FindApplicationBar(control);
						}
					}

					if (_applicationBar != null)
					{
						break;
					}
				}
			}
		} 

		#endregion Implementation Methods

	}

}
