using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Web.UI.HtmlControls;
using System.Web.UI.Design;
using System.Drawing.Design;

using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Represents a Ribbon Group box for containing Ribbon Buttons.
	/// </summary>
	[
	Designer(typeof(RibbonGroupDesigner)),
	ToolboxData("<{0}:RibbonGroup runat=\"server\"></{0}:RibbonGroup>"),
	ParseChildren(true)
	]
	public class RibbonGroup : RibbonControl, IPostBackEventHandler
	{

		#region Fields

		private ITemplate _placeHolder = null;
		private ControlCollection _placeHolderControls = null;
		private bool _showDialogLauncher = false;
		private string _dialogLauncherScript = string.Empty;
		private string _dialogLauncherToolTip = string.Empty;
		private bool _autoPostBack = false;
		private string _collapsedImageUrl = string.Empty;
		private int _collapseSequence = 0;
		private DropDownMenuAlignment _dropDownMenuAlignment = DropDownMenuAlignment.Center;

		#endregion Fields

		#region Constructors
		
		/// <summary>
		/// Initializes a new instance of the RibbonGroup class.
		/// </summary>
		public RibbonGroup() : this(string.Empty)
		{
		} 

		/// <summary>
		/// Initializes a new instance of the RibbonGroup class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		public RibbonGroup(string id) : this(id, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the RibbonGroup class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="text">
		/// The group text.
		/// </param>
		public RibbonGroup(string id, string text)
		{
			// Shared State
			// 0 - Text
			// 1 - ExpandedWidth
			// 2 - CollapsedWidth
			// 3 - Expanded
			// 4 - Open
			this.SharedState.Add(new SharedStateItem("Text", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("ExpandedWidth", typeof(int), 0));
			this.SharedState.Add(new SharedStateItem("CollapsedWidth", typeof(int), 0));
			this.SharedState.Add(new SharedStateItem("Expanded", typeof(bool), true));
			this.SharedState.Add(new SharedStateItem("Open", typeof(bool), false));

			this.ID = id;
			this.Text = text;
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets or sets the content of the group.
		/// </summary>
		[
		DefaultValue(null),
		PersistenceMode(PersistenceMode.InnerProperty), 
		Browsable(false),
		TemplateInstance(TemplateInstance.Single)
		]
		public ITemplate PlaceHolder
		{
			get
			{
				return _placeHolder;
			}
			set
			{
				_placeHolder = value;
			}
		}

		/// <summary>
		/// Gets or sets the collection of controls to be added to the place holder at run time.
		/// Design-time controls are added first then run-time.
		/// </summary>
		[
		DefaultValue(null),
		Browsable(false)
		]
		public ControlCollection PlaceHolderControls
		{
			get
			{
				if (_placeHolderControls == null)
				{
					_placeHolderControls = new ControlCollection(this);
				}

				return _placeHolderControls;
			}
		}

		/// <summary>
		/// Gets or sets the text to display in the group title.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the text to display in the group title."),
		Category("Appearance")
		]
		public string Text
		{
			get
			{
				string text = Convert.ToString(this.SharedState["Text"].Value);

				return text;
			}
			set
			{
				this.SharedState["Text"].Value = value;
			}
		}

		/// <summary>
		/// Gets or sets whether the dialog launcher button is dislayed.
		/// </summary>
		/// </summary>
		[
		DefaultValue(false),
		Description("Specifies whether the dialog launcher button is dislayed."),
		Category("Behavior")
		]
		public bool ShowDialogLauncher
		{
			get
			{
				return _showDialogLauncher;
			}
			set
			{
				_showDialogLauncher = value;
			}
		}

		/// <summary>
		/// Gets or sets a tool tip to be displayed over the dialog launcher button.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the tool tip to display for the dialog launcher button."),
		Category("Behavior")
		]
		public string DialogLauncherToolTip
		{
			get
			{
				return _dialogLauncherToolTip;
			}
			set
			{
				_dialogLauncherToolTip = value;
			}
		}

		/// <summary>
		/// Gets or set the client code to execute when the dialog launcher button is clicked.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the client code to execute when the dialog laucher button is clicked."),
		Category("Behavior")
		]
		public virtual string OnClientDialogLauncherClick
		{
			get
			{
				return _dialogLauncherScript;
			}
			set
			{
				_dialogLauncherScript = value;
			}
		}

		/// <summary>
		/// Gets or sets whether clicking the dialog launcher button causes a post back.
		/// This must be explicitly set to cause a post back even if an event handler has 
		/// been defined.
		/// </summary>
		[
		DefaultValue(false),
		Description("Specifies whether clicking the dialog launcher button causes a post back."),
		Category("Behavior")
		]
		public bool AutoPostBack
		{
			get
			{
				return _autoPostBack;
			}
			set
			{
				_autoPostBack = value;
			}
		}

		/// <summary>
		/// Gets or sets the width (in pixels) of the group when expanded.
		/// </summary>
		[
		DefaultValue(0),
		Description("Specifies the width (in pixels) of the group when expanded."),
		Category("Layout")
		]
		public int ExpandedWidth
		{
			get
			{
				int width = Convert.ToInt32(this.SharedState["ExpandedWidth"].Value);

				return width;
			}
			set
			{
				this.SharedState["ExpandedWidth"].Value = value;
			}
		}

		/// <summary>
		/// Gets or sets the image to display when collapsed. 16x16 pixels.
		/// </summary>
		[
		Editor(typeof(ImageUrlEditor), typeof(UITypeEditor)),
		Description("Specifies the image to display when collapsed. 16x16 pixels."),
		Category("Appearance")
		]
		public string CollapsedImageUrl
		{
			get
			{
				if (string.IsNullOrEmpty(_collapsedImageUrl))
				{
					_collapsedImageUrl = "~/Office2007Client/Images/Empty.gif";
				}

				return _collapsedImageUrl;
			}
			set
			{
				if (string.IsNullOrEmpty(value))
				{
					value = "~/Office2007Client/Images/Empty.gif";
				}

				value = ResolveUrl(value);
				_collapsedImageUrl = value;
			}
		}

		/// <summary>
		/// Gets or sets the order in which the group will be collapsed when the browser is resized.
		/// 0 first, 1 next, 2 next, etc.
		/// </summary>
		[
		DefaultValue(0),
		Description("Specifies the order in which the group will be collapsed when the browser is resized."),
		Category("Behavior")
		]
		public int CollapseSequence
		{
			get
			{
				return _collapseSequence;
			}
			set
			{
				_collapseSequence = value;
			}
		}

		/// <summary>
		/// Gets or sets the width (in pixels) of the group when collapsed.
		/// </summary>
		[
		DefaultValue(0),
		Description("Specifies the width (in pixels) of the group when collapsed."),
		Category("Layout")
		]
		public int CollapsedWidth
		{
			get
			{
				int width = Convert.ToInt32(this.SharedState["CollapsedWidth"].Value);

				return width;
			}
			set
			{
				this.SharedState["CollapsedWidth"].Value = value;
			}
		}

		/// <summary>
		/// Gets or sets where the drop down arrow is positioned when the group is collapsed.
		/// </summary>
		[
		DefaultValue(typeof(DropDownMenuAlignment), "Center"),
		Description("Specifies where the drop down arrow is positioned when the group is collapsed."),
		Category("Appearance")
		]
		public DropDownMenuAlignment DropDownMenuAlignment
		{
			get
			{
				return _dropDownMenuAlignment;
			}
			set
			{
				_dropDownMenuAlignment = value;
			}
		}  

		#endregion Properties

		#region Methods

		/// <summary>
		/// Sets text - allows use by control and designer.
		/// </summary>
		public override void FinalizeRender()
		{
			base.FinalizeRender();

			SetWidth();
			SetText();
			SetDialogLaucherButtonState();
			SetExpanded();
			SetCollapsedImage();
			SetDropDownMenu();

			if (!this.DesignMode)
			{
				RenderClientOnDialogLauncherClick();
				RenderClientOnCollapsedClick();
			}
		}

		#endregion Methods

		#region Events

		/// <summary>
		/// Raises the DialogLauncherClick event.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected virtual void OnDialogLauncherClick(EventArgs e)
		{
			if (this.DialogLauncherClick != null)
			{
				DialogLauncherClick(this, e);
			}
		}

		/// <summary>
		/// Occurs when the dialog launcher button is clicked.
		/// </summary>
		public event EventHandler DialogLauncherClick;

		#endregion Events

		#region Implementation Properties

		/// <summary>
		/// Gets the control's tag type.
		/// </summary>
		protected override HtmlTextWriterTag TagKey
		{
			get
			{
				return HtmlTextWriterTag.Div;
			}
		}

		/// <summary>
		/// Gets whether the group is expanded or collapsed. Because of the way the 
		/// Ribbon is designed you should not manually change this value. It is intended to 
		/// reflect the state set on the client only.
		/// </summary>
		protected virtual bool Expanded
		{
			get
			{
				bool expanded = Convert.ToBoolean(this.SharedState["Expanded"].Value);

				return expanded;
			}
		}

		/// <summary>
		/// Gets whether the group is open when collapsed. Because of the way the 
		/// Ribbon is designed you should not manually change this value. It is intended to 
		/// reflect the state set on the client only.
		/// </summary>
		protected virtual bool Open
		{
			get
			{
				bool open = Convert.ToBoolean(this.SharedState["Open"].Value);

				return open;
			}
		}

		#endregion Implementation Properties

		#region Implementation Methods

		/// <summary>
		/// Ensure child controls are available for referencing in the page load.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			EnsureChildControls();
		}

		/// <summary>
		/// Ensure we add run-time controls so they are available for event handler re-attachment.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);

			if (!this.DesignMode)
			{
				AddRuntimeControls();
			}
		}

		/// <summary>
		/// Creates the control's layout.
		/// </summary>
		protected override void CreateChildControls()
		{
			ClearControls();

			this.Attributes.Add("class", "ribbonGroupWrapper ribbonPart");

			// Collapsed

			if (!this.DesignMode)
			{
				HtmlGenericControl collapsedDiv = new HtmlGenericControl("div");
				collapsedDiv.ID = "C";
				collapsedDiv.Attributes.Add("class", "ribbonGroupContainerCollapsed ribbonPart");
				collapsedDiv.Attributes.Add("title", this.ToolTip);

				this.Controls.Add(collapsedDiv);

				HtmlGenericControl collapsedStartDiv = new HtmlGenericControl("div");
				collapsedStartDiv.EnableViewState = false;
				collapsedStartDiv.ID = "CStart";
				collapsedStartDiv.Attributes.Add("class", "ribbonGroupCollapsedStart ribbonPart");

				collapsedDiv.Controls.Add(collapsedStartDiv);

				HtmlGenericControl collapsedItemDiv = new HtmlGenericControl("div");
				collapsedItemDiv.EnableViewState = false;
				collapsedItemDiv.ID = "CItem";
				collapsedItemDiv.Attributes.Add("class", "ribbonGroupCollapsedItem ribbonPart");

				collapsedDiv.Controls.Add(collapsedItemDiv);

				HtmlGenericControl collapsedImageDiv = new HtmlGenericControl("div");
				collapsedImageDiv.EnableViewState = false;
				collapsedImageDiv.ID = "CImageContainer";
				collapsedImageDiv.Attributes.Add("class", "ribbonGroupCollapsedImageContainer ribbonPart");

				collapsedItemDiv.Controls.Add(collapsedImageDiv);

				HtmlImage collapsedImage = new HtmlImage();
				collapsedImage.EnableViewState = false;
				collapsedImage.ID = "CImage";
				collapsedImage.Attributes.Add("class", "ribbonGroupCollapsedImage ribbonPart");

				collapsedImageDiv.Controls.Add(collapsedImage);

				HtmlGenericControl collapsedTextDiv = new HtmlGenericControl("div");
				collapsedTextDiv.EnableViewState = false;
				collapsedTextDiv.ID = "CText";
				collapsedTextDiv.Attributes.Add("class", "ribbonGroupCollapsedText ribbonPart");

				collapsedItemDiv.Controls.Add(collapsedTextDiv);

				HtmlGenericControl collapsedDropDownDiv = new HtmlGenericControl("div");
				collapsedDropDownDiv.EnableViewState = false;
				collapsedDropDownDiv.ID = "CDropDownMenuImage";

				collapsedItemDiv.Controls.Add(collapsedDropDownDiv);

				HtmlGenericControl collapsedEndDiv = new HtmlGenericControl("div");
				collapsedEndDiv.EnableViewState = false;
				collapsedEndDiv.ID = "CEnd";
				collapsedEndDiv.Attributes.Add("class", "ribbonGroupCollapsedEnd ribbonPart");

				collapsedDiv.Controls.Add(collapsedEndDiv);
			}

			// Expanded 

			HtmlGenericControl expandedDiv = new HtmlGenericControl("div");
			expandedDiv.ID = "E";
			expandedDiv.Attributes.Add("class", "ribbonGroupContainer ribbonPart");
			expandedDiv.Attributes.Add("title", this.ToolTip);

			this.Controls.Add(expandedDiv);

			Table mainTable = new Table();
			mainTable.CellPadding = 0;
			mainTable.CellSpacing = 0;

			expandedDiv.Controls.Add(mainTable);

			TableRow mainRow = new TableRow();
			mainTable.Rows.Add(mainRow);
			
			TableCell startCell = new TableCell();
			mainRow.Cells.Add(startCell);

			HtmlGenericControl startDiv = new HtmlGenericControl("div");
			startDiv.EnableViewState = false;
			startDiv.ID = "EStart";
			startDiv.Attributes.Add("class", "ribbonGroupStart ribbonPart");

			startCell.Controls.Add(startDiv);

			TableCell middleCell = new TableCell();
			middleCell.Attributes.Add("nowrap", "nowrap");
			mainRow.Cells.Add(middleCell);

			HtmlGenericControl middleDiv = new HtmlGenericControl("div");
			middleDiv.EnableViewState = false;
			middleDiv.ID = "EItem";
			middleDiv.Attributes.Add("class", "ribbonGroupItem ribbonPart");

			middleCell.Controls.Add(middleDiv);

			Table placeHolderTable = new Table();
			placeHolderTable.CellPadding = 0;
			placeHolderTable.CellSpacing = 0;

			TableRow placeHolderRow = new TableRow();
			placeHolderTable.Rows.Add(placeHolderRow);

			TableCell placeHolderCell = new TableCell();
			placeHolderCell.ID = "EPlaceHolder";
			placeHolderCell.Attributes.Add("nowrap", "nowrap");
			placeHolderCell.CssClass = "ribbonGroupPlaceHolder ribbonPart";

			placeHolderRow.Cells.Add(placeHolderCell);

			if (this.PlaceHolder != null)
			{
				Control control = new Control();
				this.PlaceHolder.InstantiateIn(control);
				placeHolderCell.Controls.Add(control);
			}

			TableRow textRow = new TableRow();
			placeHolderTable.Rows.Add(textRow);

			TableCell textCell = new TableCell();
			textRow.Cells.Add(textCell);

			HtmlGenericControl containerDiv = new HtmlGenericControl("div");
			containerDiv.Attributes.Add("class", "ribbonGroupTextContainer");
			textCell.Controls.Add(containerDiv);

			HtmlGenericControl dialogLauncherDiv = new HtmlGenericControl("div");
			dialogLauncherDiv.ID = "EDialogLauncher";
			dialogLauncherDiv.Attributes.Add("class", "ribbonGroupDialogLauncher");

			containerDiv.Controls.Add(dialogLauncherDiv);

			HtmlGenericControl textDiv = new HtmlGenericControl("div");
			textDiv.ID = "EText";
			textDiv.Attributes.Add("class", "ribbonGroupText ribbonPart");

			containerDiv.Controls.Add(textDiv);

			middleDiv.Controls.Add(placeHolderTable);

			TableCell endCell = new TableCell();
			mainRow.Controls.Add(endCell);

			HtmlGenericControl endDiv = new HtmlGenericControl("div");
			endDiv.EnableViewState = false;
			endDiv.ID = "EEnd";
			endDiv.Attributes.Add("class", "ribbonGroupEnd ribbonPart");

			endCell.Controls.Add(endDiv);
		}

		/// <summary>
		/// Create client script to cause a post back or execute client side java script.
		/// </summary>
		protected virtual void RenderClientOnDialogLauncherClick()
		{
			if (this.ShowDialogLauncher)
			{
				string postBackReference = string.Empty;
				string clientCode = string.Empty;

				HtmlGenericControl dialogLauncherDiv = FindControl("EDialogLauncher") as HtmlGenericControl;

				if (this.AutoPostBack)
				{
					PostBackOptions options = new PostBackOptions(this);
					options.AutoPostBack = true;
					postBackReference = this.Page.ClientScript.GetPostBackEventReference(options);
				}

				if (!string.IsNullOrEmpty(this.OnClientDialogLauncherClick))
				{
					clientCode = this.OnClientDialogLauncherClick;
				}

				if ((!string.IsNullOrEmpty(clientCode)) || (!string.IsNullOrEmpty(postBackReference)))
				{
					string code = clientCode + postBackReference;

					// Embedded JavaScript
					dialogLauncherDiv.Attributes.Add("onclick", code);
				}
			}
		}

		/// <summary>
		/// Create client script to cause a post back or execute client side java script.
		/// </summary>
		protected virtual void RenderClientOnCollapsedClick()
		{
			HtmlGenericControl collapsedDiv = FindControl("C") as HtmlGenericControl;

			// Embedded JavaScript - Named dependency
			string code = this.ClientMemberName + ".collapsedClicked(event);";

			// Embedded JavaScript
			collapsedDiv.Attributes.Add("onclick", code);
		}

		/// <summary>
		/// Sets the internal width of the group.
		/// </summary>
		protected virtual void SetWidth()
		{
			Unit expandedWidth = this.Width;

			if (!this.DesignMode)
			{
				this.Width = Unit.Empty;
				expandedWidth = Unit.Empty;

				if (this.ExpandedWidth != 0)
				{
					expandedWidth = new Unit(this.ExpandedWidth);
				}
			}

			if (!expandedWidth.IsEmpty)
			{
				int innerWidth = 0;

				if (expandedWidth.Value > 4)
				{
					innerWidth = Convert.ToInt32(expandedWidth.Value) - 5;
				}

				if (this.DesignMode)
				{
					TableCell placeHolderCell = FindControl("EPlaceHolder") as TableCell;

					placeHolderCell.Style.Add("width", innerWidth.ToString() + "px");
				}
				else
				{
					HtmlGenericControl expandedItemDiv = FindControl("EItem") as HtmlGenericControl;

					expandedItemDiv.Style.Add("width", innerWidth.ToString() + "px");
				}
			}

			if (!this.DesignMode)
			{
				HtmlGenericControl collapsedDiv = FindControl("C") as HtmlGenericControl;
				HtmlGenericControl collapsedContentDiv = FindControl("CItem") as HtmlGenericControl;

				collapsedDiv.Style.Add(HtmlTextWriterStyle.Width, this.CollapsedWidth.ToString() + "px");
				collapsedContentDiv.Style.Add(HtmlTextWriterStyle.Width, (this.CollapsedWidth - 4).ToString() + "px");
			}
		}

		/// <summary>
		/// Sets the group's text.
		/// </summary>
		protected virtual void SetText()
		{
			HtmlGenericControl expandedTextDiv = FindControl("EText") as HtmlGenericControl;

			if (expandedTextDiv != null)
			{
				expandedTextDiv.InnerText = this.Text;
			}

			HtmlGenericControl collapsedTextDiv = FindControl("CText") as HtmlGenericControl;

			if (collapsedTextDiv != null)
			{
				collapsedTextDiv.InnerText = this.Text;
			}
		}


		/// <summary>
		/// Sets the groups's collapsed image.
		/// </summary>
		protected virtual void SetCollapsedImage()
		{
			HtmlImage image = this.FindControl("CImage") as HtmlImage;

			if (image != null)
			{
				image.Src = ResolveUrl(this.CollapsedImageUrl);
			}
		}

		/// <summary>
		/// Shows or hides the dialog launcher button. 
		/// </summary>
		private void SetDialogLaucherButtonState()
		{
			HtmlGenericControl dialogLauncherDiv = FindControl("EDialogLauncher") as HtmlGenericControl;
			dialogLauncherDiv.Visible = this.ShowDialogLauncher;
			dialogLauncherDiv.Attributes.Add("title", this.DialogLauncherToolTip);
		}

		/// <summary>
		/// Show the group in expanded or collapsed mode.
		/// </summary>
		private void SetExpanded()
		{
			const string visible = "block";
			const string hidden = "none";

			HtmlGenericControl expandedDiv = FindControl("E") as HtmlGenericControl;
			HtmlGenericControl collapsedDiv = FindControl("C") as HtmlGenericControl;

			expandedDiv.Style.Add(HtmlTextWriterStyle.Display, (this.Expanded ? visible : hidden));
			
			// At design-time the collapsed DIV is not generated so ignore it if not available
			if (collapsedDiv != null)
			{
				collapsedDiv.Style.Add(HtmlTextWriterStyle.Display, (this.Expanded ? hidden : visible));
			}
		}

		/// <summary>
		/// Sets whether the drop down image is visible.
		/// </summary>
		protected virtual void SetDropDownMenu()
		{
			HtmlGenericControl dropDownDiv = FindControl("CDropDownMenuImage") as HtmlGenericControl;

			if (dropDownDiv != null)
			{
				dropDownDiv.Attributes.Add("class", "ribbonGroupDropDownMenuArrow ribbonPart");

				string style = null;

				switch (this.DropDownMenuAlignment)
				{
					case DropDownMenuAlignment.Left:
						style = "margin-left: 4px; margin-right: auto;";
						break;

					case DropDownMenuAlignment.Center:
						style = "margin-left: auto; margin-right: auto;";
						break;

					case DropDownMenuAlignment.Right:
						style = "margin-left: auto; margin-right: 4px;";
						break;
				}

				dropDownDiv.Attributes.Add("style", style);
			}
		}

		/// <summary>
		/// Adds controls added at run-time before rendering.
		/// </summary>
		private void AddRuntimeControls()
		{
			TableCell placeHolderCell = FindControl("EPlaceHolder") as TableCell;

			foreach (Control control in this.PlaceHolderControls)
			{
				placeHolderCell.Controls.Add(control);
			}
		}

		#endregion Implementation Methods

		#region IPostBackEventHandler Members

		/// <summary>
		/// Raises the click event;
		/// </summary>
		/// <param name="eventArgument">
		/// The event data.
		/// </param>
		public void RaisePostBackEvent(string eventArgument)
		{
			OnDialogLauncherClick(EventArgs.Empty);
		}

		#endregion IPostBackEventHandler Members

	}

}
