using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Drawing;
using System.Web.UI.HtmlControls;

namespace RussellMason.Web.UI
{

	/// <summary>
	/// Base web control that allows data to be stored in a hidden field much the same 
	/// as ViewState except it allows both client and server to access the data.
	/// Shared state values should be added in the derived class' constructor to ensure
	/// it has been correctly initialized when control properties are accessed.
	/// </summary>
	public class SharedStateWebControl : WebControl
	{

		#region Fields

		private bool _enableSharedState = true;
		private SharedStateBag _sharedState = null;
		private bool _generateClientMember = false;
		private string _clientMemberName = string.Empty;

		#endregion Fields

		#region Contructors

		/// <summary>
		/// Initializes a new instance of the SharedStateWebControl.
		/// </summary>
		public SharedStateWebControl()
		{
		} 

		#endregion Contructors

		#region Properties

		/// <summary>
		/// Gets or set whether shared state is enabled. Defaults to true. 
		/// This value should be set in the contructor before state loading
		/// and saving.
		/// </summary>
		[
		DefaultValue(true),
		Category("Behavior")
		]
		public bool EnableSharedState
		{
			get
			{
				return _enableSharedState;
			}
			set
			{
				_enableSharedState = value;
			}
		}

		/// <summary>
		/// Gets or sets whether a client-side object is created. The object ID
		/// is used as the class name.
		/// </summary>
		[
		DefaultValue(false),
		Category("Behavior")
		]
		public virtual bool GenerateClientMember
		{
			get
			{
				return _generateClientMember;
			}
			set
			{
				_generateClientMember = value;
			}
		}

		/// <summary>
		/// Gets or sets the name of the variable to use when generating a client object instance.
		/// When GenerateClientMember is true the default name will be in the format _controlID (camel case),
		/// </summary>
		[
		DefaultValue(""),
		Category("Behavior")
		]
		public string ClientMemberName
		{
			get
			{
				return _clientMemberName;
			}
			set
			{	
				_clientMemberName = value;
			}
		}

		#endregion Properties

		#region Methods

		/// <summary>
		/// Allows common code to be executed prior to rendering from the control or 
		/// designer.
		/// </summary>
		public virtual void FinalizeRender()
		{
		}

		#endregion Methods

		#region Implementation Properties

		/// <summary>
		/// Gets or sets the shared state. If not explicitly set a defaut SharedStateBag 
		/// is used. This should only be set if you have created a derived SharedStateBag 
		/// for special serialization/deserialization.
		/// <para />
		/// Shared state is loaded on page load and save on pre-rendering.
		/// </summary>
		protected virtual SharedStateBag SharedState
		{
			get
			{
				if (_sharedState == null)
				{
					_sharedState = new SharedStateBag(this);
				}

				return _sharedState;
			}
			set
			{
				if (value != null)
				{
					if (value.Parent != this)
					{
						string message =
							"The parent of the shared state bag must be the same as the control it is added to.";

						throw new ArgumentException(message, "SharedState");
					}
				}

				_sharedState = value;
			}
		}

		/// <summary>
		/// Gets the size (in pixels) of the specified text using the given font.
		/// </summary>
		/// <param name="text">
		/// The text to get the size of.
		/// </param>
		/// <param name="font">
		/// The font to use.
		/// </param>
		/// <returns>
		/// The size of the text.
		/// </returns>
		protected Size MeasureString(string text, Font font)
		{
			float emSize = Convert.ToSingle(font.Size + 0);
			Font stringFont = new Font(font.Name, emSize);
			Bitmap bitmap = new Bitmap(8, 8);
			Graphics graphics = Graphics.FromImage(bitmap);

			SizeF size = graphics.MeasureString(text, stringFont);
			graphics.Dispose();
			
			return size.ToSize();
		}

		/// <summary>
		/// Gets the size (in pixels) of the specified text using the given font and wrapped
		/// to 2 lines. The result is the minimum width required when warpped over 2 lines.
		/// </summary>
		/// <param name="text">
		/// The text to get the size of.
		/// </param>
		/// <param name="font">
		/// The font to use.
		/// </param>
		/// <returns>
		/// The size of the text.
		/// </returns>
		protected Size MeasureWrappedString(string text, Font font)
		{
			Size minimumWidthSize = Size.Empty;
			string[] lines = GetWrappedTextCombinations(text);

			foreach (string line in lines)
			{
				Size size = MeasureString(line, font);

				if ((minimumWidthSize == Size.Empty) || (size.Width <minimumWidthSize.Width))
				{
					minimumWidthSize = size;
				}
			}

			return minimumWidthSize;
		}

		/// <summary>
		/// Returns all combinations of the specified text so it uses 2 lines.
		/// </summary>
		/// <param name="text">
		/// The text to wrap.
		/// </param>
		/// <returns>
		/// All combinations of the wrapped tex.
		/// </returns>
		private string[] GetWrappedTextCombinations(string text)
		{
			string[] wrappedTextCombinations = new string[0];

			if (text == null)
			{
				text = string.Empty;
			}

			List<int> breaks = new List<int>();

			for (int index = 0; index < text.Length; index++)
			{
				char lastCharacter = '\0';
				char character = text[index];

				if ((character == ' ') && (lastCharacter != ' '))
				{
					breaks.Add(index);
				}

				lastCharacter = character;
			}

			if (breaks.Count == 0)
			{
				wrappedTextCombinations = new string[] { text };
			}
			else
			{
				List<string> lines = new List<string>();

				for (int index = 0; index < breaks.Count; index++)
				{
					// Add to empty to ensure trim works on zero length strings
					string left = (string.Empty + text.Substring(0, breaks[index])).Trim();  
					string right = (string.Empty + text.Substring(breaks[index])).Trim();

					string wrappedText = left + Environment.NewLine + right;
					lines.Add(wrappedText);
				}

				wrappedTextCombinations = lines.ToArray();
			}

			return wrappedTextCombinations;
		}

		#endregion Implementation Properties

		#region Implementation Methods

		/// <summary>
		/// Loads shared state after controls have been initialized but before the control is loaded.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			if (this.EnableSharedState)
			{
				SharedStateHiddenField stateField = new SharedStateHiddenField();
				stateField.ID = this.SharedState.SharedStateFieldID;
				stateField.EnableViewState = false;

				this.Controls.Add(stateField);
			}

			if (_clientMemberName == string.Empty)
			{
				this.ClientMemberName = "_" + char.ToLower(this.ID[0]) + this.ID.Substring(1);
			}
		}

		/// <summary>
		/// Loads shared state when the control is loaded.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnLoad(EventArgs e)
		{
			LoadSharedState();

			base.OnLoad(e);
		}

		/// <summary>
		/// Saves shared state before the control has been saved.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnPreRender(EventArgs e)
		{
			if (this.EnableSharedState)
			{
				SaveSharedState();		
			}

			FinalizeRender();

			base.OnPreRender(e);
		}

		/// <summary>
		/// Clears all controls except the hidden field that contains shared state.
		/// You should always use this method in CreateChildControls to remove controls
		/// rather than Controls.Clear(). If you do not you will remove the stared state
		/// control and the control will not function correctlty and unexpected errors
		/// thrown.
		/// </summary>
		protected virtual void ClearControls()
		{
			for (int index = this.Controls.Count - 1; index >= 0; index--)
			{
				Control control = this.Controls[index];

				if (!(control is SharedStateHiddenField))
				{
					this.Controls.Remove(control);
				}
			}
		}

		/// <summary>
		/// Gets the client ID of the control with the specified ID. If the control does not
		/// exist the specified ID is returned unchanged.
		/// </summary>
		/// <param name="source">
		/// </param>
		/// <param name="id">
		/// The server ID of the control.
		/// </param>
		/// <returns>
		/// The client ID of the control if available; otherwise the passed value.
		/// </returns>
		protected virtual string GetClientID(Control source, string id)
		{
			string clientId = id;

			if ((source != null) && (!string.IsNullOrEmpty(id)))
			{
				Control control = ControlFinder.FindUniqueControl(source, id);

				if (control != null)
				{
					clientId = control.ClientID;
				}
			}

			return clientId;
		}

		/// <summary>
		/// Adds client member script to create a java script variable.
		/// </summary>
		/// <param name="writer">
		/// The writer to write to.
		/// </param>
		protected override void RenderContents(HtmlTextWriter writer)
		{
			base.RenderContents(writer);

			if ((!this.DesignMode) && (this.GenerateClientMember))
			{
				RegisterClientMemberScript(writer);
			}
		}

		/// <summary>
		/// Registes a client script to create a instance of a client side object. The object 
		/// must have a name the same as the server side class. The contructor must take a
		/// single parameter, clientID.
		/// </summary>
		protected virtual void RegisterClientMemberScript(HtmlTextWriter writer)
		{
			if (string.IsNullOrEmpty(this.ID))
			{
				string message = "The control must have an ID in order to create a client member variable.";

				throw new InvalidOperationException(message);
			}

			if (string.IsNullOrEmpty(this.ClientMemberName))
			{
				string message = "The control must have a client member name assigned.";

				throw new InvalidOperationException(message);
			}

			string typeName = this.GetType().Name;

			// Embedded JavaScript - Named dependency
			string script = "<script language=\"javascript\" type=\"text/javascript\">" + 
				this.ClientMemberName + " = new " +  typeName + "(\"" + this.ClientID + "\");" +
				"</script>";

			writer.Write(script);
		}

		/// <summary>
		/// Loads the shared state. By default this just calls the SharedState's 
		/// Load method.
		/// </summary>
		protected virtual void LoadSharedState()
		{
			this.SharedState.Load();
		}

		/// <summary>
		/// Saves the shared state. By default this just calls the SharedState's
		/// Save method.
		/// </summary>
		protected virtual void SaveSharedState()
		{
			this.SharedState.Save();
		} 

		#endregion Implementation Methods

	}

}
